/*
 * Copyright (c) 2018 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import (
	"bytes"
	"testing"
)

var varnishCluster = ShardCluster{
	Nodes: []Service{
		Service{
			Name:      "varnish-8445d4f7f-z2b9p",
			Addresses: []Address{{"172.17.0.12", 80}},
		},
		Service{
			Name:      "varnish-8445d4f7f-k22dn",
			Addresses: []Address{{"172.17.0.13", 80}},
		},
		Service{
			Name:      "varnish-8445d4f7f-ldljf",
			Addresses: []Address{{"172.17.0.14", 80}},
		},
	},
	Probe: Probe{
		Timeout:   "2s",
		Interval:  "5s",
		Initial:   "2",
		Window:    "8",
		Threshold: "3",
	},
	MaxSecondaryTTL: "5m",
}

func TestShardTemplate(t *testing.T) {
	var buf bytes.Buffer
	gold := "shard.golden"

	if err := shardTmpl.Execute(&buf, varnishCluster); err != nil {
		t.Error("cluster template Execute():", err)
		return
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for self-sharding does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}

func TestPrimaryOnlyShardTemplate(t *testing.T) {
	var buf bytes.Buffer
	gold := "primaryonly_shard.golden"

	varnishCluster.PrimaryOnly = true
	if err := shardTmpl.Execute(&buf, varnishCluster); err != nil {
		t.Error("cluster template Execute():", err)
		return
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for primary-only self-sharding does "+
			"not match gold file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}

func TestGetSrc(t *testing.T) {
	gold := "ingress_shard.golden"
	cafeSpec.ShardCluster = varnishCluster
	cafeSpec.ShardCluster.PrimaryOnly = false
	src, err := cafeSpec.GetSrc()
	if err != nil {
		t.Error("Spec.GetSrc():", err)
		return
	}
	ok, err := cmpGold([]byte(src), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL from GetSrc() does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", src)
		}
	}
}

func TestShardByURL(t *testing.T) {
	var buf bytes.Buffer
	gold := "shard_by_url.golden"

	varnishCluster.PrimaryOnly = true
	varnishCluster.By = URL
	if err := shardTmpl.Execute(&buf, varnishCluster); err != nil {
		t.Error("cluster template Execute():", err)
		return
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for self-sharding does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}

func TestShardByKey(t *testing.T) {
	var buf bytes.Buffer
	gold := "shard_by_key.golden"

	varnishCluster.PrimaryOnly = true
	varnishCluster.By = Key
	varnishCluster.Key = "req.http.Host"
	if err := shardTmpl.Execute(&buf, varnishCluster); err != nil {
		t.Error("cluster template Execute():", err)
		return
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for self-sharding does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}

func TestShardByDigest(t *testing.T) {
	var buf bytes.Buffer
	gold := "shard_by_digest.golden"

	varnishCluster.PrimaryOnly = true
	varnishCluster.By = Blob
	varnishCluster.Key = "req.http.Host"
	varnishCluster.Algo = Sha3_512
	if err := shardTmpl.Execute(&buf, varnishCluster); err != nil {
		t.Error("cluster template Execute():", err)
		return
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL for self-sharding does not match gold "+
			"file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}
